@@ -9,15 +9,15 @@ INCLUDES = """
 """
 
 TYPES = """
-typedef ... CONF;
 """
 
 FUNCTIONS = """
 void OPENSSL_config(const char *);
-void OPENSSL_no_config(void);
 """
 
 MACROS = """
+/* This is a macro in 1.1.0 */
+void OPENSSL_no_config(void);
 """
 
 CUSTOMIZATIONS = """
